import jarray, junit, types
useUnittest = True
try:
  import unittest
except:
  useUnittest = False
from org.apache.maven.surefire.testset import SurefireTestSet
from org.apache.maven.surefire.report import ReportEntry
from org.python.core import PyException

class JythonTestSet(SurefireTestSet):
  def __init__(self, testClass, testNames):
    self.classUnderTest = testClass
    self.testNames = testNames

  def getTestCount(self):
    return len(self.testNames)
  
  def getName(self):
    return self.classUnderTest.__module__ + '.' + self.classUnderTest.__name__

  def execute(self, repManager, classLoader):
    for name in self.testNames:
      testCase = self.classUnderTest(name)
      if useUnittest and isinstance(testCase, unittest.TestCase):
        testCase = PyToJTestCase(testCase)
      result = junit.framework.TestResult()
      result.addListener(JythonTestReporter(repManager))
      testCase.run(result)
    
class JythonTestReporter(junit.framework.TestListener):
  def __init__(self, reportMgr):
    self.reportMgr = reportMgr
    self.hadFailure = False

  def addError(self, test, throwable):
    self.reportMgr.testError(self.makeEntry(test, throwable))
    self.hadFailure = True

  def addFailure(self, test, assertionFailedError):
    self.reportMgr.testFailed(self.makeEntry(test, assertionFailedError))
    self.hadFailure = True

  def endTest(self, test):
    if not self.hadFailure:
      self.reportMgr.testSucceeded(self.makeEntry(test))
    
  def startTest(self, test):
    self.reportMgr.testStarting(self.makeEntry(test))

  def makeEntry(self, test, throwable=None):
    name = test.getName()
    if isinstance(test, PyToJTestCase):
      test = test._pyTestCase
    if throwable:
      stackTraceWriter = JythonTracebackWriter('%s' % test.__class__,
                                              name,
                                              throwable,
                                              PyException)
      return ReportEntry(test,
                         name,
                         throwable.toString(),
                         stackTraceWriter)
    else:
      return ReportEntry(test, name, '%s' % test.__class__)

def getTestCaseNames(testCaseClass):#Stolen from unittest.py
      testFnNames = filter(lambda n: n.startswith('test'),
                           dir(testCaseClass))
      for baseclass in testCaseClass.__bases__:
          for testFnName in getTestCaseNames(baseclass):
              if testFnName not in testFnNames:  # handle overridden methods
                  testFnNames.append(testFnName)
      testFnNames.sort()
      return testFnNames

def loadTestSets(testmodule):
  testSets = []
  for attr in [getattr(testmodule, name) for name in dir(testmodule)]:
    if hasattr(attr, '__bases__') and hasattr(attr, '__module__'):
        if issubclass(attr, junit.framework.TestCase):
          testSets.append(JythonTestSet(attr, getTestCaseNames(attr)))
        elif useUnittest and issubclass(attr, unittest.TestCase):
            testSets.append(JythonTestSet(attr, getTestCaseNames(attr)))
  return jarray.array(testSets, SurefireTestSet)

if useUnittest:#only define the unittest wrappers if unittest is available
  #Both wrappers came from http://aspn.activestate.com/ASPN/Mail/Message/Jython-users/1576852
  class JToPyTestResult(unittest.TestResult):
    """Adapts a junit TestResult to be a unittest.TestResult.  A
       unitttest.TestCase expects a unittest.TestResult.  When we adapt the
       TestCase to run in JUnit, JUnit expects results reported in its
       TestResult object.  Here we delegate the unittest.TestResult methods
       to the corresponding methods of the adapted junit TestResult."""

    def __init__(self,juTestResult):
      self._juTestResult=juTestResult

    def startTest(self,test):
      "Delegates to the wrapped junit TestResult method"
      # The test argument is a unittest.TestCase.  Wrap it before sending
      # it to the junit TestResult.
      self._juTestResult.startTest(PyToJTestCase(test))

    def stopTest(self,test):
      "Delegates to the wrapped junit TestResult endTest method"
      # The test argument is a unittest.TestCase.  Wrap it before sending
      # it to the junit TestResult.
      self._juTestResult.endTest(PyToJTestCase(test))

    def addError(self, test, err):
      "Delegates to the wrapped junit TestResult method"
      # The test argument is a unittest.TestCase.  Wrap it before sending
      # it to the junit TestResult.
      # The err argument is a tuple as returned by sys.exc_info.  The
      # exception object is the second (index 1) member of the tuple.
      # The Junit TestResult expects a java.lang.Throwable.  But it might
      # be a Python exception, which will cause a TypeError.  In that case,
      # create a Java exception with data from the Python exception as the
      # detail message.
      reason = err[1]
      try:
        self._juTestResult.addError(PyToJTestCase(test),reason)
      except TypeError:
        from org.python.core import PyException
        reason = PyException(err[0], err[1], err[2])
        self._juTestResult.addError(PyToJTestCase(test),reason)


    def addFailure(self, test, err):
      "Delegates to the wrapped junit TestResult method"
      # The test argument is a unittest.TestCase.  Wrap it before sending
      # it to the junit TestResult.
      # The err argument is a tuple as returned by sys.exc_info.  The
      # exception object is the second (index 1) member of the tuple.
      self._juTestResult.addFailure(PyToJTestCase(test),err[1])

    def wasSuccessful(self):
      "Delegates to the wrapped junit TestResult method"
      return self._juTestResult.wasSuccessful()

    def stop(self):
      "Delegates to the wrapped junit TestResult method"
      self_juTestResult.stop()


  class PyToJTestCase(junit.framework.TestCase):
    """Adapts a unittest.TestCase to be a junit TestCase so that 
       the unittest.TestCase can be run by JUnit or the JUnit Ant task"""

    def __init__(self, pyTestCase):
      junit.framework.TestCase.__init__(self,"runTest")
      self._pyTestCase = pyTestCase
      self._pyTestCase.failureException=junit.framework.AssertionFailedError

    def getName(self):
      """Returns the shortDescription of the wrapped unittest.TestCase or,
         if that is None, the methodName of the wrapped TestCase."""
      desc=self._pyTestCase.shortDescription()
      # The short description is likely to be None unless the test developer
      # provided a doc string for the test method.  If it is None, then
      # parse the str result to find the test method name.
      if desc is None:
        desc=str(self._pyTestCase).split(' ')[0]
      return desc

    def setUp(self):
      "Delegate to wrapped unittest.TestCase"
      self._pyTestCase.setUp()

    def tearDown(self):
      "Delegate to wrapped unittest.TestCase"
      self._pyTestCase.tearDown()

    def run(self, testresult):
      "Delegate to wrapped unittest.TestCase"
      # Adapt the junit test result and send it to the unittest.TestCase.
      self._pyTestCase.run(JToPyTestResult(testresult))

    def equals(self, other):
      "Ensure that multiple wrappers of the same unittest.TestCase are equal"
      return self._pyTestCase is other._pyTestCase

    def hashCode(self):
      "Ensure that multiple wrappers of the same unittest.TestCase hash the same"
      return hash(self._pyTestCase)
